#!/usr/bin/perl
# parsing the prcs commands results, used by prcs- friends

# return () if no .prj file found in current directory; return project name and
# version number if succeeded
sub get_default_project {
  my (@projects, $project, $version, $junk);
  # get project name
  @projects=map{ glob($_) } "*.prj";
  if ( $#projects>0 ) { die "More than one .prj files presented in current directory"; }
  elsif ( $#projects==-1) { return (); } 

  ($project, $junk)=split(/\./, $projects[0]);

  # get the working version
  open (PRJ, "$project.prj");
  while (<PRJ>) {
    if (/^\(Project-Version\s+[-\w]+\s+(\w+)\s+([0-9.]+)\)/) {
      $version=$1.".".$2; 
      close (PRJ);
      return ($project, $version);
    }
  }                                     
}

# return files and its MD5sums in a project for a specific version
sub remote_md5sum_hash {
  my ($host, $project, $ver)=@_;
  my %MD5s=();

  open(MD5, "rsh $host prcs info --plain-format -f -L -r $ver $project|") || die "prcs failed";
  while(<MD5>) {
    next if (/^$project\s+/); 
    if ( /^($project\/)?([^\s]+)\s+MD5=(\w+)\s+/ ) { $MD5s{$2}=$3; }
  }
  return %MD5s;
}

# return files and its MD5sums in a project for a specific version
sub md5sum_hash {
  my ($project, $ver)=@_;
  my %MD5s=();

  open(MD5, "prcs info --plain-format -L -r $ver $project|") || die "prcs failed";
  while(<MD5>) {
    next if (/^$project\s+/); 
    if ( /^($project\/)?([^\s]+)\s+MD5=(\w+)\s+/ ) { $MD5s{$2}=$3; }
  }
  return %MD5s;
}

# produce md5sum for a file
sub gen_md5_for_a_file {
  open (F, "md5sum @_|");
  while (<F>) {
    chop;
    ($md, $file)=split(" ", $_);
  }  
  return $md;
}

# produce md5sum for files
sub gen_md5 {
  open (F, "md5sum @_|");
  while (<F>) {
    chop;
    ($md, $file)=split(" ", $_);
    $md5{$file}=$md; 
  }  
  return %md5;
}

# compare two hash tables of md5s of files
sub compare {
  my $from=@_[0]; 
  my $to=@_[1]; 
  my $verbose=@_[2];
  my %md_from=%$from;
  my %md_to=%$to;
  my $key;

  foreach $key (sort keys %md_from) {
    if ( exists $md_to{$key} ) {
      if ( $md_from{$key} eq $md_to{$key} ) {
        printf "%-40s%s", $key, "Up-to-date\n" if $verbose; 
      }
      else {
        printf "%-40s%s", $key, "Modified\n"; 
      }
      delete $md_to{$key};
    }
    else { printf "%-40s%s", $key, "Removed\n"; }
  }

  foreach $key (keys %md_to) {
    printf "%-40s%s", $key, "Added\n";
  }                 
}

# return history for one file in a project
sub file_history {
  my ($project, $file)=@_;
  my @vers=();

  open(INFO, "prcs info --plain-format -L --sort=date $project|") || die "prcs failed";
  while(<INFO>) {
    if (/^$project\s+/) {
        chop;
        ($version, $weekday, $day, $month, $year, $time, $user) =
        (split(' ', $_, 10))[1..6,9];
    }
    if (/^$file\s+MD5=(\w+)/) { 
      push @vers, [$version, $user, $day." ".$month." ".$year, $1];
    }
  }                                          

  return @vers;
} 

return 1;
